@@ -1,8 +1,7 @@
 <?php
 /*
- ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2010-2017 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -527,7 +526,7 @@ function debug_wizard() {
 				$issue_line = $issues[0];
 			}
 
-			$issue_title = implode($issues, '<br/>');
+			$issue_title = implode('<br/>',$issues);
 
 			$user = db_fetch_cell_prepared('SELECT username
 				FROM user_auth
@@ -725,7 +724,7 @@ function debug_view() {
 		array(
 			'name' => 'convert_name',
 			'title' => __('Were we able to convert the title?'),
-			'value' => get_data_source_title($check['datasource'])
+			'value' => html_escape(get_data_source_title($check['datasource']))
 		),
 		array(
 			'name' => 'rrd_match',
