@@ -124,7 +124,8 @@ $cacti_version_codes = array(
 	'1.2.9'  => '0102',
 	'1.2.10'  => '0102',
 	'1.2.11'  => '0102',
-	'1.2.12'  => '0102'
+	'1.2.12'  => '0102',
+	'1.2.13'  => '0102',
 );
 
 $messages = array(
@@ -249,7 +250,7 @@ $messages = array(
 		'message' => __('You must select at least one object from the list.'),
 		'level' => MESSAGE_LEVEL_ERROR),
 	41 => array(
-		'message' => __('Device Template updated.  Remember to Sync Templates to push all changes to Devices that use this Device Template.'),
+		'message' => __('Device Template updated.  Remember to Sync Devices to push all changes to Devices that use this Device Template.'),
 		'level' => MESSAGE_LEVEL_INFO),
 	42 => array(
 		'message' => __('Save Successful. Settings replicated to Remote Data Collectors.'),
