@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2019 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -22,7 +22,7 @@
  +-------------------------------------------------------------------------+
 */
 
-function run_data_query($host_id, $snmp_query_id) {
+function run_data_query($host_id, $snmp_query_id, $automation = false, $force = false) {
 	global $config, $input_types;
 
 	if (read_config_option('data_source_trace') == 'on') {
@@ -34,6 +34,8 @@ function run_data_query($host_id, $snmp_query_id) {
 		return false;
 	}
 
+	add_orphan_support();
+
 	/* don't run/rerun the query if the host is down, or disabled */
 	$status = db_fetch_row_prepared('SELECT status, disabled, poller_id
 		FROM host
@@ -172,7 +174,7 @@ function run_data_query($host_id, $snmp_query_id) {
 
 	/* recalculate/change sort order */
 	$local_data = db_fetch_assoc_prepared('SELECT dl.id AS local_data_id, dl.host_id,
-		dl.snmp_query_id, dl.snmp_index,
+		dl.snmp_query_id, dl.snmp_index, dl.orphan,
 		did.data_input_field_id, did.data_template_data_id,
 		did.value AS query_index, "' . $old_sort_field . '" AS sort_field
 		FROM data_local AS dl
@@ -189,8 +191,10 @@ function run_data_query($host_id, $snmp_query_id) {
 		array($snmp_query_id, $host_id));
 
 	$changed_ids = array();
-	$removed_ids = array();
+	$orphaned_ids = array();
 	if (cacti_sizeof($local_data)) {
+		query_debug_timer_offset('data_query', __('Found %s Local Data ID\'s to Verify', cacti_sizeof($local_data)));
+
 		foreach($local_data as $data_source) {
 			// Just in case there is a forced type from the data source page
 			$forced_type = false;
@@ -286,7 +290,8 @@ function run_data_query($host_id, $snmp_query_id) {
 					query_debug_timer_offset('data_query', __('Index Change Detected! CurrentIndex: %s, PreviousIndex: %s', $current_index, $data_source['query_index']));
 
 					db_execute_prepared('UPDATE data_local
-						SET snmp_index = ?
+						SET snmp_index = ?,
+						orphan = 0
 						WHERE id = ?',
 						array($current_index, $data_source['local_data_id']));
 
@@ -302,16 +307,11 @@ function run_data_query($host_id, $snmp_query_id) {
 
 					// Set the index to Null, note that the Data Source still has the value
 					db_execute_prepared('UPDATE data_local
-						SET snmp_index = ""
+						SET orphan = 1
 						WHERE id = ?',
 						array($data_source['local_data_id']));
 
-					// Remove this item from the poller cache as it will cause wranings
-					db_execute_prepared('DELETE FROM poller_item
-						WHERE local_data_id = ?',
-						array($data_source['local_data_id']));
-
-					$removed_ids[] = $data_source['local_data_id'];
+					$orphaned_ids[] = $data_source['local_data_id'];
 				}
 			}
 
@@ -330,15 +330,30 @@ function run_data_query($host_id, $snmp_query_id) {
 			}
 		}
 
-		if (cacti_sizeof($changed_ids) || sizeof($removed_ids)) {
-			query_debug_timer_offset('data_query', __('Remapping Graphs to their new Indexes'));
+		query_debug_timer_offset('data_query', __('Verification of %s Local Data ID\'s Complete', cacti_sizeof($local_data)));
+
+		if (cacti_sizeof($changed_ids) || cacti_sizeof($orphaned_ids)) {
+			query_debug_timer_offset('data_query', __('Found Changed %s and %s Orphaned Local Data ID\'s to Re-map.', cacti_sizeof($changed_ids), cacti_sizeof($orphaned_ids)));
 			data_query_remap_indexes($changed_ids);
-			data_query_remap_indexes($removed_ids);
+			data_query_remap_indexes($orphaned_ids);
+			query_debug_timer_offset('data_query', __('Done remapping Graphs to their new Indexes'));
 		}
 
-		/* update title cache for graph and data source */
-		update_data_source_title_cache_from_host($host_id);
-		update_graph_title_cache_from_host($host_id);
+		if ((cacti_sizeof($changed_ids) || cacti_sizeof($orphaned_ids)) && !$force) {
+			/* update title cache for graph and data source */
+			update_data_source_title_cache_from_host($host_id, $snmp_query_id, array_merge($changed_ids, $orphaned_ids));
+			query_debug_timer_offset('data_query', __('Done updating Data Source Title Cache'));
+
+			update_graph_title_cache_from_host($host_id, $snmp_query_id, array_merge($changed_ids, $orphaned_ids));
+			query_debug_timer_offset('data_query', __('Done updating Graph Title Cache'));
+		} elseif ($force) {
+			/* update title cache for graph and data source */
+			update_data_source_title_cache_from_host($host_id, $snmp_query_id);
+			query_debug_timer_offset('data_query', __('Done updating Data Source Title Cache'));
+
+			update_graph_title_cache_from_host($host_id, $snmp_query_id);
+			query_debug_timer_offset('data_query', __('Done updating Graph Title Cache'));
+		}
 	}
 
 	query_debug_timer_offset('data_query', __('Index Association with Local Data complete'));
@@ -347,7 +362,7 @@ function run_data_query($host_id, $snmp_query_id) {
 
 	/* update the auto reindex cache */
 	if (cacti_sizeof($changed_ids)) {
-		query_debug_timer_offset('data_query', __('Update Re-Index Cache complete. There were ' . sizeof($changed_ids) . ' index changes, and ' . sizeof($removed_ids) . ' removed indexes.'));
+		query_debug_timer_offset('data_query', __('Update Re-Index Cache complete. There were ' . sizeof($changed_ids) . ' index changes, and ' . sizeof($orphaned_ids) . ' orphaned indexes.'));
 
 		/* update the poller cache */
 		update_poller_cache_from_query($host_id, $snmp_query_id, $changed_ids);
@@ -356,11 +371,12 @@ function run_data_query($host_id, $snmp_query_id) {
 		query_debug_timer_offset('data_query', __('No Index Changes Detected, Skipping Re-Index and Poller Cache Re-population'));
 	}
 
-	if (cacti_sizeof($removed_ids)) {
+	if (cacti_sizeof($orphaned_ids) &&
+		(isset($query_array['index_orphan_removal']) && $query_array['index_orphan_removal'] == 'true')) {
 		$poller_ids = array_rekey(
 			db_fetch_assoc_prepared('SELECT DISTINCT poller_id
 				FROM poller_item
-				WHERE local_data_id IN (' . implode(', ', $removed_ids) . ')'),
+				WHERE local_data_id IN (' . implode(', ', $orphaned_ids) . ')'),
 			'poller_id', 'poller_id'
 		);
 
@@ -369,9 +385,6 @@ function run_data_query($host_id, $snmp_query_id) {
 				api_data_source_cache_crc_update($poller_id);
 			}
 		}
-
-		db_execute_prepared('DELETE FROM poller_item
-			WHERE local_data_id IN (' . implode(', ', $removed_ids) . ')');
 	}
 
 	if ($config['poller_id'] == 1) {
@@ -382,7 +395,7 @@ function run_data_query($host_id, $snmp_query_id) {
 		api_plugin_hook_function('run_data_query', array('host_id' => $host_id, 'snmp_query_id' => $snmp_query_id));
 		query_debug_timer_offset('data_query', __('Plugin hooks complete'));
 	} elseif ($config['connection'] == 'online') {
-		poller_push_reindex_data_to_main($host_id, $snmp_query_id);
+		poller_push_reindex_data_to_poller($host_id, $snmp_query_id);
 
 		automation_execute_data_query($host_id, $snmp_query_id);
 		query_debug_timer_offset('data_query', __('Automation Execution for Data Query complete'));
@@ -401,9 +414,39 @@ function run_data_query($host_id, $snmp_query_id) {
 		}
 	}
 
+	// Remove orphans only if the data query expressly calls for it
+	if (cacti_sizeof($orphaned_ids) &&
+		(isset($query_array['index_orphan_removal']) && $query_array['index_orphan_removal'] == 'true')) {
+		data_query_remove_disabled_items($orphaned_ids);
+	}
+
 	return (isset($result) ? $result : true);
 }
 
+function data_query_remove_disabled_items($orphaned_ids) {
+	if (cacti_sizeof($orphaned_ids)) {
+		db_execute_prepared('DELETE FROM poller_item
+			WHERE local_data_id IN (' . implode(', ', $orphaned_ids) . ')');
+
+		db_execute_prepared('DELETE FROM poller_output_boost
+			WHERE local_data_id IN (' . implode(', ', $orphaned_ids) . ')');
+
+		$archive_tables = array_rekey(
+			db_fetch_assoc('SELECT TABLE_NAME
+				FROM information_schema.TABLES
+				WHERE TABLE_NAME LIKE "poller_output_boost_arch%"'),
+			'TABLE_NAME', 'TABLE_NAME'
+		);
+
+		if (cacti_sizeof($archive_tables)) {
+			foreach($archive_tables as $table) {
+				db_execute_prepared("DELETE FROM $table
+					WHERE local_data_id IN (' . implode(', ', $orphaned_ids) . ')", false);
+			}
+		}
+	}
+}
+
 function query_check_suitable($new_sort_field, $old_sort_field, $host_id, $snmp_query_id) {
 	if ($new_sort_field == $old_sort_field) {
 		query_debug_timer_offset('data_query', __('Checking for Sort Field change.  No changes detected.'));
@@ -623,6 +666,8 @@ function query_script_host($host_id, $snmp_query_id) {
 
 	debug_log_insert_section_end('data_query');
 
+	$empty_types = array();
+
 	/* set an array to host all updates */
 	$output_array = array();
 
@@ -636,6 +681,7 @@ function query_script_host($host_id, $snmp_query_id) {
 			$script_data_array = exec_into_array($script_path);
 
 			if (!cacti_sizeof($script_data_array) && $field_name == $sort_field) {
+				$empty_types[] = $field_name;
 				query_debug_timer_offset('data_query', __('Sort field returned no data for field name %s, skipping', $field_name));
 			} else {
 				debug_log_insert('data_query', __('Executing script query \'%s\'', $script_path));
@@ -659,7 +705,7 @@ function query_script_host($host_id, $snmp_query_id) {
 	}
 
 	if (cacti_sizeof($output_array)) {
-		data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, $output_array);
+		data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, $output_array, $empty_types);
 	}
 
 	return true;
@@ -812,6 +858,8 @@ function query_snmp_host($host_id, $snmp_query_id) {
 
 	$fields_processed = array();
 
+	$empty_types = array();
+
 	rewrite_snmp_enum_value(null);
 
 	foreach ($snmp_queries['fields'] as $field_name => $field_array) {
@@ -1035,6 +1083,7 @@ function query_snmp_host($host_id, $snmp_query_id) {
 						$snmp_data[$d['oid']] = $d['value'];
 					}
 				} elseif ($field_name == $sort_field) {
+					$empty_types[] = $field_name;
 					query_debug_timer_offset('data_query', __('Sort field returned no data for OID[%s], skipping.', $field_array['oid']));
 					continue;
 				}
@@ -1042,6 +1091,7 @@ function query_snmp_host($host_id, $snmp_query_id) {
 				$snmp_data = cacti_snmp_session_walk($session, $field_array['oid']);
 
 				if (!cacti_sizeof($snmp_data) && $field_name == $sort_field) {
+					$empty_types[] = $field_name;
 					query_debug_timer_offset('data_query', __('Sort field returned no data for OID[%s], skipping.', $field_array['oid']));
 					continue;
 				}
@@ -1116,7 +1166,7 @@ function query_snmp_host($host_id, $snmp_query_id) {
 						$oid = $field_array['oid'] .  '.' . $parse_value;
 
 						/* rewrite octet strings */
-						if (preg_match('/^\d{1,3}(\.\d{1,3}) {2,}$/', $parse_value)) {
+						if (preg_match('/^\d{1,3}(\.\d{1,3}){2,}$/', $parse_value)) {
 							$octets = explode('.', $parse_value);
 							$size = array_shift($octets);
 
@@ -1174,7 +1224,7 @@ function query_snmp_host($host_id, $snmp_query_id) {
 	$session->close();
 
 	if (cacti_sizeof($output_array)) {
-		data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, $output_array);
+		data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, $output_array, $empty_types);
 	}
 
 	return true;
@@ -1205,7 +1255,7 @@ function data_query_ctype_print_unicode($value) {
 	return preg_match($pattern, $value);
 }
 
-function data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, &$output_array) {
+function data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, &$output_array, &$empty_types) {
 	/* set all fields present value to 0, to mark the outliers when we are all done */
 	db_execute_prepared('UPDATE host_snmp_cache
 		SET present = 0
@@ -1252,8 +1302,23 @@ function data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, &$ou
 		db_execute($sql_prefix . $buffer . $sql_suffix);
 	}
 
+	if (cacti_sizeof($empty_types)) {
+		foreach($empty_types as $field) {
+			db_execute_prepared('UPDATE host_snmp_cache
+				SET present = 1
+				WHERE host_id = ?
+				AND snmp_query_id = ?
+				AND field_name = ?',
+				array($host_id, $snmp_query_id, $field));
+		}
+	}
+
 	/* remove stale records from the host cache */
-	db_execute_prepared("DELETE FROM host_snmp_cache WHERE host_id = ? AND snmp_query_id = ? AND present='0'", array($host_id, $snmp_query_id));
+	db_execute_prepared("DELETE FROM host_snmp_cache
+		WHERE host_id = ?
+		AND snmp_query_id = ?
+		AND present='0'",
+		array($host_id, $snmp_query_id));
 }
 
 /* data_query_rewrite_indexes - returns array of rewritten indexes
@@ -1627,7 +1692,8 @@ function update_data_source_data_query_cache($local_data_id, $host_id = '', $dat
 		/* set the index to the new index */
 		db_execute_prepared('UPDATE data_local
 			SET snmp_query_id = ?,
-			snmp_index = ?
+			snmp_index = ?,
+			orphan = 0
 			WHERE id = ?',
 			array($data_query_id, $current_index, $local_data_id));
 
@@ -2210,3 +2276,9 @@ function api_data_query_errors($snmp_query_graph_id, $post) {
 	return $errors;
 }
 
+function add_orphan_support() {
+	if (!db_column_exists('data_local', 'orphan')) {
+		db_execute('ALTER TABLE data_local ADD COLUMN orphan tinyint unsigned NOT NULL default "0" AFTER snmp_index');
+	}
+}
+
