@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2019 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -333,6 +333,16 @@ function push_out_graph($graph_template_graph_id) {
 		return 0;
 	}
 
+	/* don't push out if you have no graphs */
+	$has_graphs = db_fetch_cell_prepared('SELECT COUNT(id)
+		FROM graph_local
+		WHERE graph_template_id = ?',
+		array($graph_template_graph['graph_template_id']));
+
+	if ($has_graphs == 0) {
+		return 0;
+	}
+
 	/* loop through each graph column name (from the above array) */
 	foreach ($struct_graph as $field_name => $field_array) {
 		/* are we allowed to push out the column? */
@@ -435,6 +445,16 @@ function push_out_graph_item($graph_template_item_id, $local_graph_id = 0) {
 		return 0;
 	}
 
+	/* don't push out if you have no graphs */
+	$has_graphs = db_fetch_cell_prepared('SELECT COUNT(id)
+		FROM graph_local
+		WHERE graph_template_id = ?',
+		array($graph_template_item['graph_template_id']));
+
+	if ($has_graphs == 0) {
+		return 0;
+	}
+
 	/* find out if any graphs actual contain this item */
 	$exists = db_fetch_assoc_prepared('SELECT id
 		FROM graph_templates_item
@@ -629,10 +649,10 @@ function retemplate_graphs($graph_template_id, $local_graph_id = 0) {
    @arg $local_graph_id - the id of the graph to change the graph template for
    @arg $graph_template_id - id the of the graph template to change to. specify '0' for no
 	graph template
-   @arg $intrusive - (true) if the target graph template has more or less graph items than
+   @arg $force - (true) if the target graph template has more or less graph items than
 	the current graph, remove or add the items from the current graph to make them equal.
 	(false) leave the graph item count alone */
-function change_graph_template($local_graph_id, $graph_template_id, $intrusive = true) {
+function change_graph_template($local_graph_id, $graph_template_id, $force = true) {
 	global $struct_graph, $struct_graph_item;
 
 	$template_data     = parse_graph_template_id($graph_template_id);
@@ -661,12 +681,14 @@ function change_graph_template($local_graph_id, $graph_template_id, $intrusive =
 		WHERE id = ?',
 		array($graph_template_id, $local_graph_id));
 
-	if ($output_type_id > 0) {
+	if ($force) {
 		$changed = true;
-	}else if (cacti_sizeof($graph_list) && $graph_template_id != $graph_list['graph_template_id']) {
+	} elseif ($output_type_id > 0) {
 		$changed = true;
-	} else {
+	} else if (cacti_sizeof($graph_list) && $graph_template_id != $graph_list['graph_template_id']) {
 		$changed = false;
+	} else {
+		$changed = true;
 	}
 
 	if ($graph_template_id == 0) {
@@ -1005,6 +1027,9 @@ function create_complete_graph_from_template($graph_template_id, $host_id, $snmp
 
 	include_once($config['library_path'] . '/data_query.php');
 
+	// Interim adding improved orphan support
+	add_orphan_support();
+
 	if (!graph_template_whitelist_check($graph_template_id)) {
 		raise_message(10);
 		return false;
@@ -1118,7 +1143,7 @@ function create_complete_graph_from_template($graph_template_id, $host_id, $snmp
 				cacti_log('Data Source values "' . implode(', ', array_values($previous_data_source)) . '"', false, 'DSTRACE');
 			}
 
-			if (sizeof($previous_data_source)) {
+			if (cacti_sizeof($previous_data_source)) {
 				$use_previous_data = create_graph_custom_data_compatible($suggested_vals, $previous_data_source);
 			} else {
 				$use_previous_data = false;
@@ -1617,7 +1642,7 @@ function create_save_graph($host_id, $form_type, $form_id1, $form_array2, $value
 		$return_array = create_complete_graph_from_template($graph_template_id, $host_id, $snmp_query_array, $values['cg']);
 
 		if ($return_array !== false) {
-			debug_log_insert('new_graphs', __('Created: %s', get_graph_title($return_array['local_graph_id'])));
+			debug_log_insert('new_graphs', __esc('Created: %s', get_graph_title($return_array['local_graph_id'])));
 
 			/* lastly push host-specific information to our data sources */
 			if (cacti_sizeof($return_array['local_data_id'])) { # we expect at least one data source associated
@@ -1637,7 +1662,7 @@ function create_save_graph($host_id, $form_type, $form_id1, $form_array2, $value
 			$return_array = create_complete_graph_from_template($graph_template_id, $host_id, $snmp_query_array, $values['sg'][$snmp_query_array['snmp_query_id']]);
 
 			if ($return_array !== false) {
-				debug_log_insert('new_graphs', __('Created: %s', get_graph_title($return_array['local_graph_id'])));
+				debug_log_insert('new_graphs', __esc('Created: %s', get_graph_title($return_array['local_graph_id'])));
 
 				/* lastly push host-specific information to our data sources */
 				if (cacti_sizeof($return_array['local_data_id'])) { # we expect at least one data source associated
